import { RecurrentConnection } from './layer/recurrent-connection';
import { IRecurrentInput, ILayer } from './layer';
import { FeedForward, IFeedForwardOptions, IFeedForwardTrainingOptions, ITrainingStatus } from './feed-forward';
import { KernelOutput, TextureArrayOutput } from 'gpu.js';
export interface IRecurrentTrainingOptions extends IFeedForwardTrainingOptions {
}
export interface IRecurrentOptions extends IFeedForwardOptions {
    hiddenLayers: Array<(inputLayer: ILayer, recurrentInput: IRecurrentInput, index: number) => ILayer>;
}
export interface IRecurrentPreppedTrainingData {
    status: ITrainingStatus;
    preparedData: KernelOutput[][];
    endTime: number;
}
export declare class Recurrent extends FeedForward {
    trainOpts: IRecurrentTrainingOptions;
    options: IRecurrentOptions;
    _outputConnection: RecurrentConnection | null;
    _layerSets: ILayer[][];
    _hiddenLayerOutputIndices: number[];
    _model: ILayer[] | null;
    constructor(options?: Partial<IRecurrentOptions & IRecurrentTrainingOptions>);
    _connectLayers(): {
        inputLayer: ILayer;
        hiddenLayers: ILayer[];
        outputLayer: ILayer;
    };
    _connectLayersDeep(): ILayer[];
    _connectHiddenLayers(previousLayer: ILayer): ILayer[];
    initialize(): void;
    initializeDeep(): void;
    run(inputs: KernelOutput[]): TextureArrayOutput;
    runInput(input: KernelOutput): KernelOutput;
    runInputs(inputs: KernelOutput[]): KernelOutput;
    train(data: KernelOutput[][], options?: Partial<IRecurrentTrainingOptions>): ITrainingStatus;
    end(): void;
    transferData(formattedData: KernelOutput[][]): KernelOutput[][];
    _prepTraining(data: KernelOutput[][], options: Partial<IRecurrentTrainingOptions>): IRecurrentPreppedTrainingData;
    _calculateTrainingError(data: KernelOutput[][]): number;
    formatData(data: Float32Array): Float32Array;
    _calculateDeltas(target: KernelOutput[]): void;
    adjustWeights(): void;
    _trainPatterns(data: KernelOutput[][]): void;
    _trainPattern(inputs: KernelOutput[], logErrorRate: boolean): KernelOutput | null;
}
//# sourceMappingURL=recurrent.d.ts.map